@@ -1,5 +1,5 @@
 /* Public API for GNU gettext PO files.
-   Copyright (C) 2003-2010, 2015-2016 Free Software Foundation, Inc.
+   Copyright (C) 2003-2010, 2014, 2023 Free Software Foundation, Inc.
    Written by Bruno Haible <bruno@clisp.org>, 2003.
 
    This program is free software: you can redistribute it and/or modify
@@ -13,11 +13,13 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
 #endif
+/* Avoid side effect of gnulib's error.h on 'struct po_error_handler'.  */
+#define _GL_NO_INLINE_ERROR
 
 /* Specification.  */
 #include "gettext-po.h"
