@@ -1,5 +1,5 @@
 /* Message list charset and locale charset handling.
-   Copyright (C) 2001-2003, 2005-2009, 2019 Free Software Foundation, Inc.
+   Copyright (C) 2001-2003, 2005-2009, 2019-2023 Free Software Foundation, Inc.
    Written by Bruno Haible <haible@clisp.cons.org>, 2001.
 
    This program is free software: you can redistribute it and/or modify
@@ -32,13 +32,16 @@
 # include <iconv.h>
 #endif
 
+#include "noreturn.h"
 #include "progname.h"
-#include "basename.h"
+#include "basename-lgpl.h"
+#include "string-desc.h"
 #include "message.h"
 #include "po-charset.h"
 #include "xstriconv.h"
 #include "xstriconveh.h"
 #include "msgl-ascii.h"
+#include "msgl-ofn.h"
 #include "xalloc.h"
 #include "xmalloca.h"
 #include "c-strstr.h"
@@ -51,11 +54,7 @@
 
 #if HAVE_ICONV
 
-static void conversion_error (const struct conversion_context* context)
-#if defined __GNUC__ && ((__GNUC__ == 2 && __GNUC_MINOR__ >= 5) || __GNUC__ > 2)
-     __attribute__ ((noreturn))
-#endif
-;
+_GL_NORETURN_FUNC static void conversion_error (const struct conversion_context* context);
 static void
 conversion_error (const struct conversion_context* context)
 {
@@ -92,6 +91,22 @@ convert_string_directly (iconv_t cd, const char *string,
   return NULL;
 }
 
+string_desc_t
+convert_string_desc_directly (iconv_t cd, string_desc_t string,
+                              const struct conversion_context* context)
+{
+  char *result = NULL;
+  size_t resultlen = 0;
+
+  if (xmem_cd_iconv (string_desc_data (string), string_desc_length (string),
+                     cd, &result, &resultlen) == 0)
+    return string_desc_new_addr (resultlen, result);
+
+  conversion_error (context);
+  /* NOTREACHED */
+  return string_desc_new_empty ();
+}
+
 static char *
 convert_string (const iconveh_t *cd, const char *string,
                 const struct conversion_context* context)
@@ -301,7 +316,7 @@ iconv_message_list_internal (message_list_ty *mlp,
         po_xerror (PO_SEVERITY_FATAL_ERROR, NULL, NULL, 0, 0, false,
                    xasprintf (_("Cannot convert from \"%s\" to \"%s\". %s relies on iconv(), and iconv() does not support this conversion."),
                               canon_from_code, canon_to_code,
-                              basename (program_name)));
+                              last_component (program_name)));
 
       context.from_code = canon_from_code;
       context.to_code = canon_to_code;
@@ -333,7 +348,7 @@ iconv_message_list_internal (message_list_ty *mlp,
           po_xerror (PO_SEVERITY_FATAL_ERROR, NULL, NULL, 0, 0, false,
                      xasprintf (_("Cannot convert from \"%s\" to \"%s\". %s relies on iconv(). This version was built without iconv()."),
                                 canon_from_code, canon_to_code,
-                                basename (program_name)));
+                                last_component (program_name)));
 #endif
     }
 
@@ -366,6 +381,15 @@ iconv_msgdomain_list (msgdomain_list_ty *mdlp,
                xasprintf (_("target charset \"%s\" is not a portable encoding name."),
                           to_code));
 
+  /* Test whether the control characters required for escaping file names with
+     spaces are present in the target encoding.  */
+  if (msgdomain_list_has_filenames_with_spaces (mdlp)
+      && !(canon_to_code == po_charset_utf8
+           || strcmp (canon_to_code, "GB18030") == 0))
+    po_xerror (PO_SEVERITY_FATAL_ERROR, NULL, NULL, 0, 0, false,
+               xasprintf (_("Cannot write the control characters that protect file names with spaces in the %s encoding"),
+                          canon_to_code));
+
   for (k = 0; k < mdlp->nitems; k++)
     iconv_message_list_internal (mdlp->item[k]->messages,
                                  mdlp->encoding, canon_to_code, update_header,
