@@ -1,10 +1,10 @@
 /* Terminal control for outputting styled text to a terminal.
-   Copyright (C) 2006-2008, 2017, 2019 Free Software Foundation, Inc.
+   Copyright (C) 2006-2008, 2017, 2019-2023 Free Software Foundation, Inc.
    Written by Bruno Haible <bruno@clisp.org>, 2019.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 3 of the License, or
+   the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
@@ -25,7 +25,6 @@
 
 #include <errno.h>
 #include <signal.h>
-#include <stdbool.h>
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
@@ -46,6 +45,7 @@
 #include "sig-handler.h"
 #include "full-write.h"
 #include "same-inode.h"
+#include "xalloc.h"
 
 #define SIZEOF(a) (sizeof(a) / sizeof(a[0]))
 
@@ -721,11 +721,11 @@ stopping_signal_handler (int sig)
 /* The signal handler for SIGCONT.
    It is reentrant.  */
 static _GL_ASYNC_SAFE void
-continuing_signal_handler (int sig)
+continuing_signal_handler (int sigcont)
 {
   int saved_errno = errno;
 
-  log_signal_handler_called (sig);
+  log_signal_handler_called (sigcont);
   update_pgrp_status ();
   /* Only do something while some output was interrupted.  */
   if (active_controller != NULL
@@ -813,7 +813,8 @@ ensure_other_signal_handlers (void)
   if (!signal_handlers_installed)
     {
       /* Install the handlers for the fatal signals.  */
-      at_fatal_signal (fatal_signal_handler);
+      if (at_fatal_signal (fatal_signal_handler) < 0)
+        xalloc_die ();
 
       #if defined SIGCONT
 
